# PALM Model

This is the PALM model source code repository. PALM is an advanced and modern meteorological model system for atmospheric and oceanic boundary-layer flows. It has been developed as a turbulence-resolving large-eddy simulation (LES) model that is especially designed for performing on massively parallel computer architectures.

PALM comes prepackages with the RRTMG [library](lib).

# Installation

First make sure to satisfy the **Software Requirements**. On Debian-based Linux Distributions this can be achieved by the following command:

``` bash
sudo apt-get install gfortran make cmake libopenmpi-dev openmpi-bin libnetcdff-dev netcdf-bin libfftw3-dev python3-pip
```

Also some additional python dependencies are needed, which can be installed using `pip`. In case you wand to use a virtual environment for these dependencies, please make sure to create one first. Afterwards you can install the python dependencies by executing the following command:

``` bash
python3 -m pip install -r share/palmtest/requirements.txt
```

Now the PALM model can be installed with the following commands (please replace `<install-prefix>` with the desired installation directory):

``` bash
bash install -p <install-prefix>
export PATH=<install-prefix>/bin:${PATH}
```

The following optional command permanently adds this installation to your bash environment:

``` bash
echo 'export PATH=<install-prefix>/bin:${PATH}' >> ~/.bashrc
```

Type `bash install -h` to get all available option of the `install` script. During installation the script also calls the respective `install` script of the RRTMG and installs it to the chosen `<install-prefix>` directory. Therefore, it is not necessary to manually install the RRTMG.

You can test your installation with the following command:

``` bash
palmtest --cases urban_environment_restart --cores 4
```

# Usage

After a successful installation the PALM executables have been linked into the directory `<install-prefix>/bin` and a default PALM configuration file can be found at `<install-prefix>/.palm.config.default`. In case you have installed the python dependencies inside a virtual environment, that environment needs to be active whenever you wand to use PALM. Next, you need to create your first PALM setup in order to start a simulation. To start a first simulation, please execute the following sequence of commands:

``` bash
mkdir -p <install-prefix>/JOBS/example_cbl/INPUT
cp packages/palm/model/tests/cases/example_cbl/INPUT/example_cbl_p3d <install-prefix>/JOBS/example_cbl/INPUT/
cd <install-prefix>
palmrun -r example_cbl -c default -a "d3#" -X 4 -v -z
```


# Changes since last svn-revision

The following lists shortly describe the main differences between this release and the last frozen revision r4901 of the old svn-repository.

## Bugfixes:

-   sky view factor restart MPI I/O: error message string was too long, initial setting of svf IO indices to zero was wrong (bugs were introduced with the last svn revision 4896 and 4898), total domain boundaries must not include the ghost layers, because SVF are not defined there, some variables extended to 64-bit INTEGERs, MPI error at end of run fixed (solves trac ticket #1269)
-   ALLOCATE statement was missing for global_end_index (restart input of surface data)
-   calculate correct maximum of plant canopy factors in case of calc_svf = .TRUE.
-   avoid unrealistic RTM output of the incoming SW radiation fluxes due to angular discretization
-   stop the model with error message when improper values for raytrace_discrete_azims and raytrace_discrete_elevs set, issue warning when few values of raytrace_discrete_azims and raytrace_discrete_elevs are set instead of the default values
-   division-by-zero with mrt_nlevels = 4
-   discretization of wall temperatures in the edges of layers modified. The approach shall have smaller discretization errors in such a situation. The change follows similar change made in LSM some time ago and it makes discretization approach in USM and LSM again consistent. Beside this, the update also improves pre-set parameters of the pavements in LSM and radiation modules and add a few minor bug fixes.
-   at east/westward-facing walls the surface-parallel velocity was not computed correctly, the v-component needs to be taken instead of the u-component (see calc_uvw_abs_v\_vgrid). The error propagated into the horizontal momentum fluxes and caused an asymmetry e.g. between north-south and east-west orientated canyons. The bug is in since the very first implementation of the topography.
-   at land- and building surfaces fluxes still contained the density. For horizontal surfaces this does not matter since the density cancels out in diffusion_s. At vertical surfaces the heat input is overestimated by about 20%. Tests for a medium-sized setup for Berlin has shown that the impact on boundary-layer structure is not significant.
-   some restart MPI-I/O errors regarding specific output quantities and 3d-arrays of type LOGICAL, MPI I/O bug with cyclic-fill when using topography in the main run
-   wrong vertical velocity removal if topography is used
-   output of perturbation pressure corrected (so far, it was normalized by the near surface density)
-   wrong shift of the coordinates of the virtual measurement (VM) locations within the output, wrong velocity interpolation in the virtual flight module
-   height levels for some profile output quantities revised
-   aerosol emission fluxes in SALSA: timestamp was not correctly selected if the simulation does not start from t=0 (i.e., restart runs)
-   lpm stochastic model: wrong calculation of the CFL-condition when the stochastic velocity components are added to the interpolated resolved velocity components, wrong calculation of stochastic term of the Weil equation
-   calculation of subsidence velocity at the first computational level (k=1)

## Changes:

-   transpose-compute overlap feature of the FFT-pressure-solver removed from the code (initialization parameter transpose_compute_overlap removed)

## New features:

-   pressure drag on vertical surfaces added as new 2d-output quantity (data_output = \'pressure_drag_x\*\', \'pressure_drag_y\*\'))
-   true air temperature at 2m added as new 2d-output quantity (data_output = \'ta_2m\*\')
-   in case of using the FFT-solver, the total number of gridpoints along x or y ( nx+1, ny+1 ) must not be multiples of npex / npey any more. In such cases, subdomains are not uniform in size any more (some of them have one grid point less than others).
-   Lagrangian particle model: simple interpolation method extended to subgrid-scale velocites
-   enable virtual flights that start with a descent
-   FFT-solver accepts non-cyclic boundary conditions along x/y. ATTENTION: this is an experimental feature an has not been rigorously tested so far.


# License

This repository is free and open-source software licensed under the [GNU GPLv3](LICENSE)
